@@ -1,7 +1,7 @@
 /* about.c - 2000/05/05 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -100,24 +100,26 @@ void Show_About_Window (void)
     gchar *description_text = 
             _(""
             "    EasyTAG is an utility for viewing, editing and writing tags of "
-            "your MP3, MP2, FLAC and OGG files. His simple and nice GTK+ interface "
-            "make tagging easier."
+            "your MP3, MP2, FLAC and OGG files. Its simple and nice GTK+ interface "
+            "makes tagging easier."
             "");
     /* Translation contributions */
     gchar *translations_thanks_text [][2]= {
-            {"    - Bastian Kleineidam ",    _("(German translation)\n")},
-            {"    - Adrian Bunk ",           _("(German translation)\n")},
-            {"    - Colin Marquardt ",       _("(German translation)\n")},
-            {"    - Philipp Thomas ",        _("(German translation)\n")},
-            {"    - Sergey Zhumatiy ",       _("(Russian translation)\n")},
-            {"    - Andrey Astafiev ",       _("(Russian translation)\n")},
-            {"    - Vincent van Adrighem ",  _("(Dutch translation)\n")},
-            {"    - Bjrn Olievier ",        _("(Dutch translation)\n")},
-            {"    - Patrik Israelsson ",     _("(Swedish translation)\n")},
-            {"    - Szel Miklos ",           _("(Hungarian translation)\n")},
-            {"    - Cappelletti Lorenzo ",   _("(Italian translation)\n")},
-            {"    - Takeshi Aihana ",        _("(Japanese translation)\n")},
-            {"    - Olexander Kunytsa ",     _("(Ukrainian translation)\n")},
+            {"    - Bastian Kleineidam ",      _("(German translation)\n")},
+            {"    - Adrian Bunk ",             _("(German translation)\n")},
+            {"    - Colin Marquardt ",         _("(German translation)\n")},
+            {"    - Philipp Thomas ",          _("(German translation)\n")},
+            {"    - Sergey Zhumatiy ",         _("(Russian translation)\n")},
+            {"    - Andrey Astafiev ",         _("(Russian translation)\n")},
+            {"    - Vincent van Adrighem ",    _("(Dutch translation)\n")},
+            {"    - Bjrn Olievier ",          _("(Dutch translation)\n")},
+            {"    - Patrik Israelsson ",       _("(Swedish translation)\n")},
+            {"    - Szel Miklos ",             _("(Hungarian translation)\n")},
+            {"    - Cappelletti Lorenzo ",     _("(Italian translation)\n")},
+            {"    - Takeshi Aihana ",          _("(Japanese translation)\n")},
+            {"    - Olexander Kunytsa ",       _("(Ukrainian translation)\n")},
+            {"    - Milan Siebenburger ",      _("(Czech translation)\n")},
+            {"    - Jaime Serrano Cartagena ", _("(Spanish translation)\n")},
             {NULL,NULL}
             };
     /* General contributions */
