@@ -1,7 +1,7 @@
 /* browser.c - 2000/04/28 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -468,7 +468,7 @@ void Browser_Tree_Node_Selected (GtkCTree *tree, GtkCTreeNode *node, gint column
          Browser_Update_Current_Path(dirnode->path);
         
         /* Display the selected path into the BrowserEntry */
-        gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(BrowserEntry)->entry),dirnode->path);
+        gtk_entry_set_text_1(BrowserEntry,dirnode->path);
 
         /* Start to read the directory */
         /* The first time 'counter' is equal to zero and if we don't want to load 
@@ -769,12 +769,12 @@ void Browser_List_Delete_File (void)
     GList   *SelectedRow = NULL;
     FileRow *filerow = NULL;
     guint    row = 0;
-    
+
     if (!ETFileList || !BrowserList || GTK_CLIST(BrowserList)->rows==0)
         return;
 
     SelectedRow = ((GtkCList*)BrowserList)->selection;
-    if (SelectedRow && SelectedRow->data!=NULL)
+    if (SelectedRow)
     {
         row = (gint)SelectedRow->data;
         filerow = gtk_clist_get_row_data(GTK_CLIST(BrowserList),row);
@@ -784,9 +784,13 @@ void Browser_List_Delete_File (void)
             {
                 // File deleted, we remove the corresponding line in the list
                 gtk_clist_remove(GTK_CLIST(BrowserList),row);
-                // Note: info of the new current file must be displayed
-                ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
-                gtk_clist_select_row(GTK_CLIST(BrowserList),row,0);
+                if (ETFileList)
+                {
+                    // Note: info of the new current file must be displayed before selecting the row
+                    ET_Display_File_Data_To_UI((ET_File *)ETFileList->data);
+                    gtk_clist_select_row(GTK_CLIST(BrowserList),
+                        (row>GTK_CLIST(BrowserList)->rows-1)?GTK_CLIST(BrowserList)->rows-1:row,0);
+                }
             }
         }
     }
@@ -863,6 +867,13 @@ GtkWidget *Create_Browser_Tree_Popup_Menu (GtkCTree *ctree)
                   (GtkSignalFunc)Browser_Popup_Menu_Handler,
                   GTK_OBJECT(BrowserPopupMenu));
 
+    MenuItem = gtk_menu_item_new_with_label(_("Run Audio Player"));
+    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Run_Audio_Player_With_File_List,NULL);
+
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+
     MenuItem = gtk_menu_item_new_with_label(_("Go to Home Directory"));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
     gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Browser_Load_Home_Directory,NULL);
@@ -911,19 +922,12 @@ GtkWidget *Create_Browser_Tree_Popup_Menu (GtkCTree *ctree)
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
     gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Browser_Tree_Rebuild,NULL); /* '..._connect_object' to send NULL parameter to Browser_Tree_Rebuild */
 
-    MenuItem = gtk_menu_item_new();
-    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-
-    MenuItem = gtk_menu_item_new_with_label(_("Run XMMS"));
-    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Run_Xmms,NULL);
-
     gtk_widget_show_all(BrowserPopupMenu);
     return BrowserPopupMenu;
 }
 GtkWidget  *Create_Browser_List_Popup_Menu (GtkCList *clist)
 {
-    GtkWidget *BrowserPopupMenu;
+    GtkWidget *BrowserPopupMenu,*BrowserPopupSubMenuSorting;
     GtkWidget *MenuItem;
 
     // Note: there is an interesting example with GtkItemFactoryEntry and gtk_item_factory_set_translate_func in XMMS sources xmms/playlistwin.c
@@ -932,49 +936,127 @@ GtkWidget  *Create_Browser_List_Popup_Menu (GtkCList *clist)
                   (GtkSignalFunc)Browser_Popup_Menu_Handler,
                   GTK_OBJECT(BrowserPopupMenu));
 
+    MenuItem = gtk_menu_item_new_with_label(_("Run Audio Player"));
+    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Run_Audio_Player_With_One_File,NULL);
+    
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Search File..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Open_Search_File_Window,NULL);
+
     MenuItem = gtk_menu_item_new_with_label(_("Delete this file"));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
     gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Browser_List_Delete_File,NULL);
     
+    MenuItem = gtk_menu_item_new_with_label(_("Reload the directory"));
+    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Browser_Reload_Directory,NULL);
+
     MenuItem = gtk_menu_item_new();
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
 
     MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by filename..."));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Sort_File_list_By_Ascending_Filename,NULL);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_FILENAME));
     
     MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by filename..."));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Sort_File_list_By_Descending_Filename,NULL);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_FILENAME));
     
     MenuItem = gtk_menu_item_new();
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
 
     MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by creation date..."));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Sort_File_List_By_Ascending_Creation_Date,NULL);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_CREATION_DATE));
     
     MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by creation date..."));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Sort_File_List_By_Descending_Creation_Date,NULL);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_CREATION_DATE));
     
     MenuItem = gtk_menu_item_new();
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
 
     MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by track number..."));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Sort_File_List_By_Ascending_Track_Number,NULL);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_TRACK_NUMBER));
     
     MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by track number..."));
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Sort_File_List_By_Descending_Track_Number,NULL);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_TRACK_NUMBER));
+
+    // Items of sub menu for other way of sorting
+    BrowserPopupSubMenuSorting = gtk_menu_new();
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by title..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_TITLE));
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by title..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_TITLE));
+    
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by artist..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_ARTIST));
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by artist..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_ARTIST));
+    
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by album..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_ALBUM));
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by album..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_ALBUM));
 
     MenuItem = gtk_menu_item_new();
-    gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
 
-    MenuItem = gtk_menu_item_new_with_label(_("Reload the directory"));
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by year..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_YEAR));
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by year..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_YEAR));
+
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by genre..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_GENRE));
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by genre..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_GENRE));
+
+    MenuItem = gtk_menu_item_new();
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list ascending by comment..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_ASCENDING_COMMENT));
+
+    MenuItem = gtk_menu_item_new_with_label(_("Sort list descending by comment..."));
+    gtk_menu_append(GTK_MENU(BrowserPopupSubMenuSorting),MenuItem);
+    gtk_signal_connect_object(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)ET_Sort_File_List_And_Update_UI,GINT_TO_POINTER(SORTING_BY_DESCENDING_COMMENT));
+
+    // Sub menu for other way of sorting
+    MenuItem = gtk_menu_item_new_with_label(_("Other sorting..."));
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(MenuItem),BrowserPopupSubMenuSorting);
     gtk_menu_append(GTK_MENU(BrowserPopupMenu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Browser_Reload_Directory,NULL);
 
     gtk_widget_show_all(BrowserPopupMenu);
     return BrowserPopupMenu;
@@ -1254,6 +1336,7 @@ GtkWidget *Create_Browser_Items (GtkWidget *parent)
      */
     BrowserEntry = gtk_combo_new();
     gtk_combo_disable_activate(GTK_COMBO(BrowserEntry));
+    gtk_combo_set_case_sensitive(GTK_COMBO(BrowserEntry),TRUE);
     /* History list */
     History_List = Load_Path_Entry_List();
     if (History_List)
@@ -1262,7 +1345,7 @@ GtkWidget *Create_Browser_Items (GtkWidget *parent)
 
     gtk_signal_connect(GTK_OBJECT(GTK_COMBO(BrowserEntry)->entry),"activate",GTK_SIGNAL_FUNC(Browser_Entry_Activated),NULL);
     gtk_box_pack_start(GTK_BOX(VerticalBox),BrowserEntry,FALSE,TRUE,0);
-    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(BrowserEntry)->entry),_("No path selected!"));
+    //gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(BrowserEntry)->entry),_("No path selected!"));
     gtk_tooltips_set_tip(Tips,GTK_COMBO(BrowserEntry)->entry,_("Enter a directory to browse."),NULL);
 
 
@@ -1279,11 +1362,10 @@ GtkWidget *Create_Browser_Items (GtkWidget *parent)
     BrowserTree = gtk_ctree_new(1,0);
     gtk_container_add(GTK_CONTAINER(ScrollWindowCTree),BrowserTree);
     gtk_ctree_set_indent(GTK_CTREE(BrowserTree),13);
-    gtk_ctree_set_line_style(GTK_CTREE(BrowserTree),GTK_CTREE_LINES_DOTTED);
-    gtk_ctree_set_expander_style(GTK_CTREE(BrowserTree),GTK_CTREE_EXPANDER_SQUARE);
+    gtk_ctree_set_line_style(GTK_CTREE(BrowserTree),    BROWSER_LINE_STYLE);
+    gtk_ctree_set_expander_style(GTK_CTREE(BrowserTree),BROWSER_EXPANDER_STYLE);
     gtk_clist_set_column_auto_resize(GTK_CLIST(BrowserTree),0,TRUE);
-    //gtk_clist_set_selection_mode (GTK_CLIST(BrowserTree),GTK_SELECTION_EXTENDED);
-    //gtk_clist_set_selection_mode (GTK_CLIST(BrowserTree),GTK_SELECTION_BROWSE);
+    //gtk_clist_set_selection_mode(GTK_CLIST(BrowserTree),GTK_SELECTION_EXTENDED);
 
     /* Create pixmaps */
     if(!opened_folder_pixmap)
@@ -1322,12 +1404,6 @@ GtkWidget *Create_Browser_Items (GtkWidget *parent)
      */
     PopupMenu = Create_Browser_Tree_Popup_Menu(GTK_CTREE(BrowserTree));
 
-    /* Open root node */
-    /*gtk_ctree_expand(GTK_CTREE(BrowserTree),RootNode);*/
-
-    /* Load current path */
-    /* Don't load it now! */
-
 
     /*
      * The ScrollWindow and the CList
@@ -1449,12 +1525,12 @@ void Browser_Open_Rename_Directory_Window (void)
     Combo = gtk_combo_new();
     gtk_box_pack_start(GTK_BOX(VBox),Combo,TRUE,TRUE,0);
     gtk_combo_set_value_in_list(GTK_COMBO(Combo),FALSE,FALSE);
+    gtk_combo_set_case_sensitive(GTK_COMBO(Combo),TRUE);
     gtk_widget_set_usize(GTK_WIDGET(Combo),300,-1);
     /* Set the directory into the combobox */
     combo_list = g_list_append(combo_list,"");
     combo_list = g_list_append(combo_list,directory_name);
-//    gtk_combo_set_popdown_strings(GTK_COMBO(Combo),combo_list);
-//*** FIX ME! When activated, can't change letter case
+    gtk_combo_set_popdown_strings(GTK_COMBO(Combo),combo_list);
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(Combo)->entry),directory_name);
     /* We attach usefull data to the combobox */
     gtk_object_set_data(GTK_OBJECT(Combo),"Parent_Directory",directory_parent);
@@ -1484,7 +1560,7 @@ void Browser_Open_Rename_Directory_Window (void)
     gtk_widget_grab_default(Button);
     gtk_signal_connect(GTK_OBJECT(Button),"clicked",(GtkSignalFunc)Destroy_Rename_Directory_Window,NULL);
 
-    gtk_widget_show_all(RenameDirectoryWindow);
+    gtk_widget_show(RenameDirectoryWindow);
 
     /* Note : the window must be shown before moving it.
      * The window is centered over the Browser Area */
@@ -1493,6 +1569,9 @@ void Browser_Open_Rename_Directory_Window (void)
     gtk_window_reposition(GTK_WINDOW(RenameDirectoryWindow),
         mw_x + ba_x + (BrowseArea->allocation.width/2)  - (RenameDirectoryWindow->allocation.width)/2,
         mw_y + ba_y + (BrowseArea->allocation.height/2) - (RenameDirectoryWindow->allocation.height)/2);
+
+    // To avoid/minimize 'flicker'
+    gtk_widget_show_all(RenameDirectoryWindow);
 }
 void Destroy_Rename_Directory_Window (void)
 {
@@ -1600,11 +1679,14 @@ void Rename_Directory (GtkObject *combobox)
         return;
     }
 
+    Destroy_Rename_Directory_Window();
+    Browser_Tree_Rebuild(new_path);
+
+    g_free(last_path);
+    g_free(new_path);
     g_free(tmp_path);
     g_free(directory_new_name);
 
-    Destroy_Rename_Directory_Window();
-    Browser_Tree_Rebuild(new_path);
     Statusbar_Message(_("Directory renamed"),TRUE);
 }
 void Rename_Directory_Window_Key_Press (GtkWidget *window, GdkEvent *event)
@@ -1627,7 +1709,10 @@ void Rename_Directory_Window_Key_Press (GtkWidget *window, GdkEvent *event)
 
 
 
-
+/*
+ * Window where is typed the name of the program to run, which
+ * receives the current directory as parameter.
+ */
 void Browser_Open_Run_Program_Window (void)
 {
     GtkWidget *Frame;
@@ -1681,6 +1766,7 @@ void Browser_Open_Run_Program_Window (void)
     Combo = gtk_combo_new();
     gtk_box_pack_start(GTK_BOX(VBox),Combo,TRUE,TRUE,0);
     gtk_combo_set_value_in_list(GTK_COMBO(Combo),FALSE,FALSE);
+    gtk_combo_set_case_sensitive(GTK_COMBO(Combo),TRUE);
     gtk_widget_set_usize(GTK_WIDGET(Combo),150,-1);
     gtk_tooltips_set_tip(Tips,GTK_COMBO(Combo)->entry,_("Enter the program to run. It will "
         "receive the current directory as parameter."),NULL);
