@@ -1,7 +1,7 @@
 /* mpeg_header.c - 2000/05/12 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include "misc.h"
 #include "mylocale.h"
 
-#include "mpg123/mpg123.h"
+#include "libmpg123/mpg123.h"
 
 
 
@@ -97,8 +97,8 @@ gboolean Mpeg_Header_Read_File_Info (gchar *filename, ET_File_Info *ETFileInfo)
 
     /* Get size of file */
     filesize = Get_File_Size(filename);
-    ETFileInfo->size       = filesize;
-    ETFileList_TotalSize     += filesize;
+    ETFileInfo->size      = filesize;
+    ETFileList_TotalSize += filesize;
 
     /*
      * This part was taken from XMMS
@@ -180,7 +180,7 @@ gboolean Mpeg_Header_Read_File_Info (gchar *filename, ET_File_Info *ETFileInfo)
 		fclose(file);
     }else
     {
-        g_print(_("ERROR while opening file: '%s' (%s)\n\a"),filename,g_strerror(errno));
+        g_print(_("ERROR while opening file: '%s' (%s).\n\a"),filename,g_strerror(errno));
         return FALSE;
     }
 
@@ -199,14 +199,16 @@ gboolean Mpeg_Header_Display_File_Info_To_UI(gchar *filename, ET_File_Info *ETFi
     gchar *time1 = NULL;
     gchar *size = NULL;
     gchar *size1 = NULL;
+    gint ln_num = sizeof(layer_names)/sizeof(layer_names[0]);
 
 
     /* MPEG, Layer versions */
     gtk_label_set_text(GTK_LABEL(VersionLabel),_("MPEG"));
+    ln_num = sizeof(layer_names)/sizeof(layer_names[0]); // Used to avoid problem with layer_names[]
     if (ETFileInfo->mpeg25)
-        text = g_strdup_printf("2.5, Layer %s",layer_names[ETFileInfo->layer-1]);
+        text = g_strdup_printf("2.5, Layer %s",(ETFileInfo->layer>=1 && ETFileInfo->layer<=ln_num)?layer_names[ETFileInfo->layer-1]:"?");
     else
-        text = g_strdup_printf("%d, Layer %s",ETFileInfo->version,layer_names[ETFileInfo->layer-1]);
+        text = g_strdup_printf("%d, Layer %s",ETFileInfo->version,(ETFileInfo->layer>=1 && ETFileInfo->layer<=ln_num)?layer_names[ETFileInfo->layer-1]:"?");
     gtk_label_set_text(GTK_LABEL(VersionValueLabel),text);
     g_free(text);
 
