@@ -1,7 +1,7 @@
 /* ogg_tag.c - 2001/11/08 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2001-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@
 #include <errno.h>
 #include <vorbis/codec.h>
 #include <vorbis/vorbisfile.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
 #include "easytag.h"
@@ -45,7 +44,9 @@
 /***************
  * Declaration *
  ***************/
- 
+
+#define MULTIFIELD_SEPARATOR " - "
+
 /* OGG fields names :
  *  - TITLE        : Track name
  *  - VERSION      : The version field may be used to differentiate multiple version of the same track title in a single collection. (e.g. remix info)
@@ -90,11 +91,14 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     vorbis_comment *vc;
     gchar *string = NULL;
     gchar *string1 = NULL;
+    guint field_num;
 
 
     if (!filename || !FileTag)
         return FALSE;
 
+    ogg_error_msg = NULL;
+
     if ( (file=fopen(filename,"rb")) == NULL )
     {
         g_print(_("ERROR while opening file: '%s' (%s).\n\a"),filename,g_strerror(errno));
@@ -105,6 +109,7 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     if ( vcedit_open(state,file) < 0 )
     {
         g_print(_("ERROR: Failed to open file: '%s' as vorbis (%s).\n"),filename,vcedit_error(state));
+        ogg_error_msg = vcedit_error(state);
         fclose(file);
         return FALSE;
     }
@@ -116,34 +121,55 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     /*********
      * Title *
      *********/
-    if ( (string = vorbis_comment_query(vc,"title",0)) != NULL && strlen(string)>0 )
+    field_num = 0;
+    while ( (string = vorbis_comment_query(vc,"title",field_num++)) != NULL )
     {
-        string = convert_from_utf8(string);
-        Strip_String(string);
-        FileTag->title = g_strdup(string);
-        g_free(string);
+        if ( strlen(string)>0 )
+        {
+            string = convert_from_utf8(string);
+            //Strip_String(string);
+            if (FileTag->title==NULL)
+                FileTag->title = g_strdup(string);
+            else
+                FileTag->title = g_strconcat(FileTag->title,MULTIFIELD_SEPARATOR,string,NULL);
+        }
+        if (string) g_free(string);
     }
 
     /**********
      * Artist *
      **********/
-    if ( (string = vorbis_comment_query(vc,"artist",0)) != NULL && strlen(string)>0 )
+    field_num = 0;
+    while ( (string = vorbis_comment_query(vc,"artist",field_num++)) != NULL )
     {
-        string = convert_from_utf8(string);
-        Strip_String(string);
-        FileTag->artist = g_strdup(string);
-        g_free(string);
+        if ( strlen(string)>0 )
+        {
+            string = convert_from_utf8(string);
+            //Strip_String(string);
+            if (FileTag->artist==NULL)
+                FileTag->artist = g_strdup(string);
+            else
+                FileTag->artist = g_strconcat(FileTag->artist,MULTIFIELD_SEPARATOR,string,NULL);
+        }
+        if (string) g_free(string);
     }
 
     /*********
      * Album *
      *********/
-    if ( (string = vorbis_comment_query(vc,"album",0)) != NULL && strlen(string)>0 )
+    field_num = 0;
+    while ( (string = vorbis_comment_query(vc,"album",field_num++)) != NULL )
     {
-        string = convert_from_utf8(string);
-        Strip_String(string);
-        FileTag->album = g_strdup(string);
-        g_free(string);
+        if ( strlen(string)>0 )
+        {
+            string = convert_from_utf8(string);
+            //Strip_String(string);
+            if (FileTag->album==NULL)
+                FileTag->album = g_strdup(string);
+            else
+                FileTag->album = g_strconcat(FileTag->album,MULTIFIELD_SEPARATOR,string,NULL);
+        }
+        if (string) g_free(string);
     }
 
     /********
@@ -152,7 +178,7 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     if ( (string = vorbis_comment_query(vc,"date",0)) != NULL && strlen(string)>0 )
     {
         string = convert_from_utf8(string);
-        Strip_String(string);
+        //Strip_String(string);
         FileTag->year = g_strdup(string);
         g_free(string);
     }
@@ -163,7 +189,7 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     if ( (string = vorbis_comment_query(vc,"tracknumber",0)) != NULL && strlen(string)>0 )
     {
         string = convert_from_utf8(string);
-        Strip_String(string);
+        //Strip_String(string);
         string1 = strchr(string,'/');
         if (NUMBER_TRACK_FORMATED)
         {
@@ -188,45 +214,56 @@ gboolean Ogg_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     /*********
      * Genre *
      *********/
-    if ( (string = vorbis_comment_query(vc,"genre",0)) != NULL && strlen(string)>0 )
+    field_num = 0;
+    while ( (string = vorbis_comment_query(vc,"genre",field_num++)) != NULL )
     {
-        string = convert_from_utf8(string);
-        Strip_String(string);
-        FileTag->genre = g_strdup(string);
-        g_free(string);
+        if ( strlen(string)>0 )
+        {
+            string = convert_from_utf8(string);
+            //Strip_String(string);
+            if (FileTag->genre==NULL)
+                FileTag->genre = g_strdup(string);
+            else
+                FileTag->genre = g_strconcat(FileTag->genre,MULTIFIELD_SEPARATOR,string,NULL);
+        }
+        if (string) g_free(string);
     }
 
     /***********
      * Comment *
      ***********/
-    if ( ((string  = vorbis_comment_query(vc,"comment",0)) != NULL && strlen(string)>0 )   // Winamp format
-      || ((string1 = vorbis_comment_query(vc,"",0))        != NULL && strlen(string1)>0) ) // Xmms format
+    field_num = 0;
+    string1 = NULL; // Cause it may be not updated into the 'while' condition
+    while ( ((string  = vorbis_comment_query(vc,"comment",field_num)) != NULL )   // Winamp format
+         || ((string1 = vorbis_comment_query(vc,"",       field_num)) != NULL ) ) // Xmms format
     {
-        if (string && strlen(string)>0 && !string1) // Contains comment to Winamp format only
+        if ( string && strlen(string)>0 ) // Contains comment to Winamp format and we prefer this format (field name defined)
         {
             string = convert_from_utf8(string);
-            Strip_String(string);
-            FileTag->comment = g_strdup(string);
-            g_free(string);
-        }else if (!string && string1 && strlen(string1)>0 ) // Contains comment to Xmms format only
+            //Strip_String(string);
+            if (FileTag->comment==NULL)
+                FileTag->comment = g_strdup(string);
+            else
+                FileTag->comment = g_strconcat(FileTag->comment,MULTIFIELD_SEPARATOR,string,NULL);
+        }else if ( string1 && strlen(string1)>0 ) // Contains comment to Xmms format only
         {
             string1 = convert_from_utf8(string1);
-            Strip_String(string1);
-            FileTag->comment = g_strdup(string1);
-            g_free(string1);
-        }else // Contains comment to the both formats
-        {
-            // We prefer winamp format (field defined) than xmms format
-            string = convert_from_utf8(string);
-            Strip_String(string);
-            FileTag->comment = g_strdup(string);
-            g_free(string);
+           // Strip_String(string1);
+            if (FileTag->comment==NULL)
+                FileTag->comment = g_strdup(string1);
+            else
+                FileTag->comment = g_strconcat(FileTag->comment,MULTIFIELD_SEPARATOR,string1,NULL);
         }
+        if (string)  g_free(string);
+        if (string1) g_free(string1);
+        string1 = NULL;
+        field_num++;
     }
 
 
     vcedit_clear(state);
     fclose(file);
+
     return TRUE;
 }
 
@@ -237,8 +274,6 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
     File_Tag *FileTag;
     gchar *filename_in;
     FILE *file_in;
-    struct stat statbuf;
-    mode_t file_permissions = 0;
     vcedit_state *state;
     vorbis_comment *vc;
     gchar *string, *string1;
@@ -249,6 +284,7 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
 
     FileTag     = (File_Tag *)ETFile->FileTag->data;
     filename_in = ((File_Name *)ETFile->FileNameCur->data)->value;
+    ogg_error_msg = NULL;
 
     /* Test to know if we can write into the file */
     if ( (file_in=fopen(filename_in,"rb"))==NULL )
@@ -257,14 +293,11 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
         return FALSE;
     }
 
-    /* Save permissions of the file (cause they may change with files on NFS) */
-    if ( stat(filename_in,&statbuf)!=-1 )
-        file_permissions = statbuf.st_mode & (S_IRWXU|S_IRWXG|S_IRWXO);
-
     state = vcedit_new_state();    // Allocate memory for 'state'
     if ( vcedit_open(state,file_in) < 0 )
     {
         g_print(_("ERROR: Failed to open file: '%s' as vorbis (%s).\n"),filename_in,vcedit_error(state));
+        ogg_error_msg = vcedit_error(state);
         fclose(file_in);
         return FALSE;
     }
@@ -355,18 +388,22 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
     // We write the comment using the "both" format
     if ( FileTag->comment )
     {
-        // Format used into xmms-1.2.5
-        string  = g_strconcat("=",FileTag->comment,NULL);
-        string1 = convert_to_utf8(string);
-        vorbis_comment_add(vc,string1);
-        g_free(string);
-        g_free(string1);
         // Format used in winamp plugin
         string = g_strconcat("comment=",FileTag->comment,NULL);
         string1 = convert_to_utf8(string);
         vorbis_comment_add(vc,string1);
         g_free(string);
         g_free(string1);
+
+        if (OGG_TAG_WRITE_XMMS_COMMENT)
+        {
+            // Format used into xmms-1.2.5
+            string  = g_strconcat("=",FileTag->comment,NULL);
+            string1 = convert_to_utf8(string);
+            vorbis_comment_add(vc,string1);
+            g_free(string);
+            g_free(string1);
+        }
     }
 
     //{gint i; for (i = 0; i < vc->comments; i++) g_print("%s -> ogg vc< '%s'\n",g_basename(filename_in),vc->user_comments[i]);}
@@ -375,7 +412,7 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
     if ( Ogg_Tag_Write_File(file_in,filename_in,state) == FALSE )
     {
         g_print(_("ERROR: Failed to write comments to file '%s' (%s).\n"),filename_in,vcedit_error(state));
-        //g_print(_("Error, tag not written (file: %s)\n"),g_basename(filename));
+        ogg_error_msg = vcedit_error(state);
         return FALSE;
     }else
     {
@@ -384,10 +421,6 @@ gboolean Ogg_Tag_Write_File_Tag (ET_File *ETFile)
 
     vcedit_clear(state);
 
-    /* Set permissions for the file, saved before */
-    if ( stat(filename_in,&statbuf)!=-1 )
-        chmod(filename_in,file_permissions);
-
     return TRUE;
 }
 
@@ -472,7 +505,7 @@ gboolean Ogg_Header_Read_File_Info (gchar *filename, ET_File_Info *ETFileInfo)
 
     if ( (file=fopen(filename,"r"))==NULL )
     {
-        g_print(_("ERROR while opening file: '%s' (%s)\n\a"),filename,g_strerror(errno));
+        g_print(_("ERROR while opening file: '%s' (%s).\n\a"),filename,g_strerror(errno));
         return FALSE;
     }
 
