@@ -1,7 +1,7 @@
 /* prefs.h - 2000/05/06 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,7 +36,8 @@ gint OptionsNoteBook_Scanner_Page_Num;    /* Contains the number of the page "Sc
 /* Common */
 GtkWidget *LoadOnStartup;
 GtkWidget *DefaultPathToMp3;
-GtkWidget *SortingFileOptionMenu;
+GtkWidget *BrowserLineStyleOptionMenu;
+GtkWidget *BrowserExpanderStyleOptionMenu;
 GtkWidget *WriteId3v1Tag;
 GtkWidget *WriteId3v2Tag;
 GtkWidget *StripTagWhenEmptyFields;
@@ -58,33 +59,16 @@ GtkWidget *PaneHandlePosition2;
 GtkWidget *PaneHandleButton2;
 
 GtkWidget *ShowHeaderInfos;
-GtkWidget *ShowCommandButtons;
-
-GtkWidget *ShowSpinButton;
-GtkWidget *ShowGoButton;
-GtkWidget *ShowFirstButton;
-GtkWidget *ShowPrevButton;
-GtkWidget *ShowNextButton;
-GtkWidget *ShowLastButton;
-GtkWidget *ShowScanButton;
-GtkWidget *ShowScanAllButton;
-GtkWidget *ShowRemoveButton;
-GtkWidget *ShowRemoveAllButton;
-GtkWidget *ShowUndoButton;
-GtkWidget *ShowUndoAllButton;
-GtkWidget *ShowRedoButton;
-GtkWidget *ShowRedoAllButton;
-GtkWidget *ShowSaveButton;
-GtkWidget *ShowSaveAllButton;
-GtkWidget *ShowQuitButton;
 GtkWidget *UseColoredTooltips;
 
 /* Misc */
 GtkWidget *ReplaceIllegalCharactersInFilename;
 GtkWidget *DateAutoCompletion;
 GtkWidget *NumberTrackFormated;
+GtkWidget *OggTagWriteXmmsComment;
+GtkWidget *SortingFileOptionMenu;
+GtkWidget *SortingFileCaseSensitive;
 GtkWidget *UseCharacterSetTranslation;
-GtkWidget *TranslationTablePath;
 GtkWidget *FileCharacterSetEntry;
 GtkWidget *UserCharacterSetEntry;
 
@@ -92,11 +76,14 @@ GtkWidget *MessageBoxPositionNone;
 GtkWidget *MessageBoxPositionCenter;
 GtkWidget *MessageBoxPositionMouse;
 
+GtkWidget *FilePlayerEntry;
+
 /* Scanner */
 GtkWidget *ConvertUnderscoreAndP20IntoSpace;
 GtkWidget *ConvertSpaceIntoUnderscore;
 GtkWidget *OverwriteTagField;
-GtkWidget *ScanWindowOnTop;
+GtkWidget *OpenScannerWindowOnStartup;
+GtkWidget *ScannerWindowOnTop;
 
 GtkWidget *SetDefaultComment;
 GtkWidget *DefaultComment;
@@ -121,7 +108,8 @@ GtkWidget *ConfirmWritePlayList;
 void Init_OptionsWindow (void);
 void Open_OptionsWindow (void);
 
-void Update_Button_Visibility_Into_Command_Bar (void);
+void File_Selection_Window_For_File      (GtkWidget *widget);
+void File_Selection_Window_For_Directory (GtkWidget *widget);
 
 
 #endif /* __PREFS_H__ */
