@@ -1,7 +1,7 @@
 /* scan.c - 2000/06/16 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #include "easytag.h"
 #include "prefs.h"
 #include "setting.h"
-#include "id3tag.h"
+#include "id3_tag.h"
 #include "bar.h"
 #include "misc.h"
 #include "et_core.h"
@@ -138,13 +138,13 @@ gchar *Scanner_Option_Menu_Items [] =
     N_("Process Fields")
 };
 /* Must have the same order than Scanner_Option_Menu_Items */
-typedef enum
+/*typedef enum
 {
-    FILL_TAG = 0,
-    RENAME_FILE,
-    PROCESS_FIELDS
+    SCANNER_FILL_TAG = 0,
+    SCANNER_RENAME_FILE,
+    SCANNER_PROCESS_FIELDS
 } Scanner_Option_Menu_Items_Value;
-
+*/
 typedef enum
 {
     LEADING_SEPARATOR = 0, /* characters before the first code */
@@ -154,8 +154,11 @@ typedef enum
     EMPTY_FIELD            /* item when entry contains no text */
 } Mask_Item_Type;
 
-typedef struct _Mask_Item Mask_Item;
-struct _Mask_Item
+/*
+ * Used into Rename File Scanner
+ */
+typedef struct _File_Mask_Item File_Mask_Item;
+struct _File_Mask_Item
 {
     Mask_Item_Type  type;
     gchar          *string;
@@ -200,8 +203,7 @@ void Mask_Editor_Clist_Key_Press (GtkWidget *widget, GdkEvent *event);
 void Mask_Editor_Create_Masks_List_From_Clist (void);
 void Mask_Editor_Clist_Load_Mask_List (GList *list);
 
-gboolean Scanner_Option_Menu_Get_Active_Menu_Item (gchar *item_text);
-void     Scanner_Option_Menu_Activate_Item (void);
+void Scanner_Option_Menu_Activate_Item (GtkWidget *widget, Scanner_Option_Menu_Items_Value item);
 
 
 
@@ -213,7 +215,10 @@ void     Scanner_Option_Menu_Activate_Item (void);
 
 void Init_ScannerWindow (void)
 {
-    ScannerWindow = (GtkWidget *)NULL;
+    ScannerWindow     = (GtkWidget *)NULL;
+    ScannerOptionMenu = (GtkWidget *)NULL;
+    SWScanButton      = (GtkWidget *)NULL;
+    SWScanAllButton   = (GtkWidget *)NULL;
 }
 
 
@@ -229,9 +234,17 @@ void Scan_Tag_With_Mask (ET_File *ETFile)
     GtkWidget *dest = NULL;
     gchar *source = NULL;
     gchar *mask = NULL;
-    gchar *separator;
     gchar *tmp;
+    gchar buf[256];
+    gchar separator[256];
+    gchar string[1024];
     gint len, i, loop=0;
+    gchar **mask_splitted;
+    gchar **file_splitted;
+    guint mask_splitted_number;
+    guint file_splitted_number;
+    guint mask_splitted_index;
+    guint file_splitted_index;
 
 
     if (!ScannerWindow || !ScannerScanTagMaskCombo || !ETFile) return;
@@ -240,14 +253,21 @@ void Scan_Tag_With_Mask (ET_File *ETFile)
     source = g_strdup(((File_Name *)((GList *)ETFile->FileNameNew)->data)->value);
     if (!mask || !source) return;
 
-    /* Remove extension of file (if found) */
+    // Remove extension of file (if found)
     tmp = strrchr(source,'.');
     for (i=0; i<ET_FILE_DESCRIPTION_SIZE; i++)
+    {
         if ( strcasecmp(tmp,ETFileDescription[i].Extension)==0 )
+        {
             *tmp = 0; //strrchr(source,'.') = 0;
+            break;
+        }
+    }
+    if (i==ET_FILE_DESCRIPTION_SIZE)
+        g_print("Tag scanner: strange..., the extension '%s' was not found into filename '%s'!\n",tmp,g_basename(source));
 
 
-    /* Replace characters into mask and filename before parsing */
+    // Replace characters into mask and filename before parsing
     if (CONVERT_UNDERSCORE_AND_P20_INTO_SPACE)
     {
         Scan_Convert_Underscore_Into_Space(mask);
@@ -262,90 +282,134 @@ void Scan_Tag_With_Mask (ET_File *ETFile)
     }
 
 
-    while ( mask!=NULL && strlen(mask)>1 )
+    // Scanner mask
+    mask_splitted = g_strsplit(mask,"/",0);
+    // Get number of arguments into 'mask_splitted'
+    for (mask_splitted_number=0;mask_splitted[mask_splitted_number];mask_splitted_number++);
+
+    // File Path
+    file_splitted = g_strsplit(source,"/",0);
+    // Get number of arguments into 'file_splitted'
+    for (file_splitted_number=0;file_splitted[file_splitted_number];file_splitted_number++);
+
+    // Set the starting position of each tab
+    if (mask_splitted_number <= file_splitted_number)
     {
-        loop++;
-
-        /*
-         * Determine first code and destinataire
-         */
-        if ( (tmp=strrchr(mask,'%'))==NULL || strlen(tmp)<2 )
-            goto Quit_Function;
-
-        dest = Scan_Return_Entry_From_Mask_Code(tmp[1]);
-        if (dest==NULL) goto Quit_Function;
+        mask_splitted_index = 0;
+        file_splitted_index = file_splitted_number - mask_splitted_number;
+    }else
+    {
+        mask_splitted_index = mask_splitted_number - file_splitted_number;
+        file_splitted_index = 0;
+    }
 
 
-        /*
-         * Delete text before code (note: mask is read from the right 
-         * to the left) into mask and source
-         */
-        if ( strlen(tmp)>2 )
+    loop = 0;
+    while ( mask_splitted[mask_splitted_index]!= NULL && file_splitted[file_splitted_index]!=NULL )
+    {
+        gchar *mask_seq = mask_splitted[mask_splitted_index];
+        gchar *file_seq = file_splitted[file_splitted_index];
+        
+        //g_print(">%d> seq '%s' '%s'\n",loop,mask_seq,file_seq);
+        while ( mask_seq && strlen(mask_seq)>0 )
         {
-            /* There is a text before... */
-            /* FIXME: be sure that the deleted text into source is the same of mask */
-            len = strlen(tmp)-2;
-            *(source + strlen(source) - len) = '\0';    /* Delete text into source... */
-            *(mask + strlen(mask) - len) = '\0';        /* Delete text into mask... */
-        }
-        *(mask + strlen(mask) - 2) = '\0';    /* Delete code into mask... */
 
-
-        /*
-         * Determine separator or text after code
-         */
-        if ( mask && strlen(mask)>0 )
-        {
-            if ( (tmp=strrchr(mask,'%'))==NULL || strlen(tmp)<2 )
+            /*
+             * Determine (first) code and destinataire
+             */
+            if ( (tmp=strchr(mask_seq,'%')) == NULL || strlen(tmp) < 2 )
             {
-                /* No code found... */
-                separator = mask;
+                break;
+            }
+            
+            // Get the target entry for this code
+            dest = Scan_Return_Entry_From_Mask_Code(tmp[1]);
+
+            /*
+             * Delete text before the code
+             */
+            if ( (len = strlen(mask_seq) - strlen(tmp)) > 0 )
+            {
+                // Get this text in 'mask_seq'
+                strncpy(buf,mask_seq,len);
+                buf[len] = 0;
+                // We remove it in 'mask_seq'
+                mask_seq = mask_seq + len;
+                // Find the same text at the begining of 'file_seq' ?
+                if ( (strstr(file_seq,buf)) == file_seq )
+                {
+                    file_seq = file_seq + len; // We remove it
+                }else
+                {
+                    g_print(_("Scan Error: can't find separator '%s' within '%s'\n"),buf,file_seq);
+                }
+            }
+
+            // Remove the current code into 'mask_seq'
+            mask_seq = mask_seq + 2;
+
+            /*
+             * Determine separator between two code or trailing text (after code)
+             */
+            if ( mask_seq && strlen(mask_seq)>0 )
+            {
+                if ( (tmp=strchr(mask_seq,'%')) == NULL || strlen(tmp) < 2 )
+                {
+                    // No more code found
+                    strcpy(separator,mask_seq);
+                    separator[strlen(mask_seq)] = 0;
+                }else
+                {
+                    len = strlen(mask_seq) - strlen(tmp);
+                    strncpy(separator,mask_seq,len);
+                    separator[len] = 0;
+                }
+
+                // Remove the current separator in 'mask_seq'
+                mask_seq = mask_seq + strlen(separator);
+
+                // Try to find the separator in 'file_seq'
+                if ( (tmp=strstr(file_seq,separator)) == NULL )
+                {
+                    g_print(_("Scan Error: can't find separator '%s' within '%s'\n"),separator,file_seq);
+                    separator[0] = 0; // Needed to avoid error when calculting 'len' below
+                }
+
+                // Get the string affected to the code (or the corresponding entry field)
+                len = strlen(file_seq) - (tmp!=NULL?strlen(tmp):0);
+                strncpy(string,file_seq,len);
+                string[len] = 0;
+
+                // Remove the current separator in 'file_seq'
+                file_seq = file_seq + strlen(string) + strlen(separator);
+
+                // We display the text affected to the code
+                Scan_Display_String_To_Ui(dest,string);
             }else
             {
-                separator = tmp + 2;
-            }
-
-            /* Try to find separator into source */
-            for (i=0;i<=strlen(source);i++)
-            {
-                tmp = source + strlen(source) - i; 
-                if ( (tmp=strstr(tmp,separator)) != NULL )
-                    break;
-            }
-            if (!tmp)
-            {
-                g_print(_("Scan Error: can't find separator '%s' within '%s'\n"),separator,source);
-                Statusbar_Message(_("Scan failed!"),TRUE);
-                goto Quit_Function;
-            }
-
-            tmp = tmp + strlen(separator);
-            Scan_Display_String_To_Ui(dest,tmp);
-
-            /* Delete separator and string into mask and source */
-            *(source + strlen(source) - strlen(separator) - strlen(tmp)) = '\0';
-            *(mask + strlen(mask) - strlen(separator)) = '\0';
-
-        }else
-        {
-            /* No more data into mask, so we suppose that we are at the limit 
-             * to a parent directory */
-            if ( (tmp=strrchr(source,'/'))!=NULL)
-            {
-                tmp = tmp + 1;
-                Scan_Display_String_To_Ui(dest,tmp);
+                // We display the remaining text, affected to the code (no more data in 'mask_seq')
+                Scan_Display_String_To_Ui(dest,file_seq);
+                
             }
         }
+        
+        // Next sequences
+        mask_splitted_index++;
+        file_splitted_index++;
+        loop++;
     }
-    Statusbar_Message(_("Tag successfully scanned..."),TRUE);
-    g_print(_("Tag successfully scanned...(%s)\n"),g_basename(((File_Name *)((GList *)ETFile->FileNameNew)->data)->value));
 
-    Quit_Function:
-        if (mask) g_free(mask);
-        if (source) g_free(source);
+    g_free(mask);
+    g_free(source);
+    g_strfreev(mask_splitted);
+    g_strfreev(file_splitted);
+
+    Statusbar_Message(_("Tag successfully scanned..."),TRUE);
+    g_print(_("Tag successfully scanned...(%s)\n"),g_basename( ((File_Name *)ETFile->FileNameNew->data)->value ) );
 }
 
 
+
 /*
  * Set the string into the 'dest' entry widget.
  * We can set the string if the widget is empty or we force it (overwrite)
@@ -464,9 +528,9 @@ gchar *Scan_Generate_New_Filename_From_Mask (gchar *mask)
     gchar *filename_new = NULL;
     gchar *filename_tmp = NULL;
     GList *new_filename_list = NULL;
-    Mask_Item *mask_item;
-    Mask_Item *mask_item_prev;
-    Mask_Item *mask_item_next;
+    File_Mask_Item *mask_item;
+    File_Mask_Item *mask_item_prev;
+    File_Mask_Item *mask_item_next;
     gint counter = 0;
 
     if (!mask) return NULL;
@@ -479,7 +543,7 @@ gchar *Scan_Generate_New_Filename_From_Mask (gchar *mask)
         // Mask contains some characters after the code ('%b__')
         if (strlen(tmp)>2)
         {
-            mask_item = g_malloc0(sizeof(Mask_Item));
+            mask_item = g_malloc0(sizeof(File_Mask_Item));
             if (counter) mask_item->type = SEPARATOR;
             else         mask_item->type = TRAILING_SEPARATOR;
             mask_item->string = g_strdup(tmp+2);
@@ -491,7 +555,7 @@ gchar *Scan_Generate_New_Filename_From_Mask (gchar *mask)
             entry_text = gtk_entry_get_text_1(source_entry);
         else
             entry_text = NULL;
-        mask_item = g_malloc0(sizeof(Mask_Item));
+        mask_item = g_malloc0(sizeof(File_Mask_Item));
         if (source_entry && entry_text && strlen(entry_text)>0)
         {
             mask_item->type = FIELD;
@@ -509,7 +573,7 @@ gchar *Scan_Generate_New_Filename_From_Mask (gchar *mask)
     // It may have some characters before the last remaining code ('__%a')
     if (mask!=NULL && strlen(mask)>0)
     {
-        mask_item = g_malloc0(sizeof(Mask_Item));
+        mask_item = g_malloc0(sizeof(File_Mask_Item));
         mask_item->type = LEADING_SEPARATOR;
         mask_item->string = g_strdup(mask);
         new_filename_list = g_list_prepend(new_filename_list,mask_item);
@@ -527,12 +591,12 @@ gchar *Scan_Generate_New_Filename_From_Mask (gchar *mask)
 
     while (new_filename_list)
     {
-        Mask_Item *mask_item = new_filename_list->data;
+        File_Mask_Item *mask_item = new_filename_list->data;
 
         if ( mask_item->type==TRAILING_SEPARATOR ) // Trailing characters of mask
         {
             // Doesn't write it if previous field is empty
-            if (new_filename_list->prev && ((Mask_Item *)new_filename_list->prev->data)->type!=EMPTY_FIELD)
+            if (new_filename_list->prev && ((File_Mask_Item *)new_filename_list->prev->data)->type!=EMPTY_FIELD)
             {
                 filename_tmp = filename_new;
                 filename_new = g_strconcat(mask_item->string,filename_new,NULL);
@@ -593,9 +657,9 @@ gchar *Scan_Generate_New_Filename_From_Mask (gchar *mask)
     {
         if (new_filename_list->data)
         {
-            if ( ((Mask_Item *)new_filename_list->data)->string )
-                g_free(((Mask_Item *)new_filename_list->data)->string);
-            g_free( (Mask_Item *)new_filename_list->data );
+            if ( ((File_Mask_Item *)new_filename_list->data)->string )
+                g_free(((File_Mask_Item *)new_filename_list->data)->string);
+            g_free( (File_Mask_Item *)new_filename_list->data );
         }
         if (!new_filename_list->prev) break;
         new_filename_list = new_filename_list->prev;
@@ -610,8 +674,7 @@ void Scan_Rename_File_Generate_Preview (void)
     gchar *filename_new = NULL;
     gchar *mask = NULL;
 
-    if (!ScannerWindow || !ScannerRenameFileMaskCombo || !RenameFilePreviewLabel 
-    || !Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE])) return;
+    if (!ScannerWindow || !ScannerRenameFileMaskCombo || !RenameFilePreviewLabel) return;
 
     mask = g_strdup(gtk_entry_get_text_1(ScannerRenameFileMaskCombo));
     filename_new = Scan_Generate_New_Filename_From_Mask(mask);
@@ -852,7 +915,7 @@ GtkWidget *Scan_Return_Entry_From_Mask_Code (gchar code)
         case 'i':    /* Ignored */
             return DummyEntry;
         default:
-            g_print("Scanner: Invalid code '%%%c' found => Scanner aborted!\a\n",code);
+            g_print("Scanner: Invalid code '%%%c' found!\n",code);
             return NULL;
     }
 }
@@ -878,7 +941,7 @@ GtkWidget *Scan_Return_Entry_From_Mask_Code (gchar code)
 #include "../pixmaps/blackwhite.xpm"
 #include "../pixmaps/forbidden.xpm"
 
-void Open_ScannerWindow (void)
+void Open_ScannerWindow (Scanner_Option_Menu_Items_Value scanner_type)
 {
     GtkWidget *ScanVBox;
     GtkWidget *HBox1, *HBox2, *HBox4, *VBox, *hbox, *vbox;
@@ -901,16 +964,23 @@ void Open_ScannerWindow (void)
     {
         //gdk_window_show(ScannerWindow->window);
         gdk_window_raise(ScannerWindow->window);
-        /*Scan_Select_Mode_And_Run_Scanner();*/
+        if (ScannerOptionMenu)
+        {
+            gtk_option_menu_set_history(GTK_OPTION_MENU(ScannerOptionMenu),scanner_type);
+            Scanner_Option_Menu_Activate_Item(NULL,scanner_type);
+        }
         return;
     }
 
+    if (scanner_type < 0)
+        scanner_type = SCANNER_FILL_TAG;
+
     /* The window */
     ScannerWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     /* Config */
     gtk_container_border_width(GTK_CONTAINER(ScannerWindow),5);
     gtk_window_set_policy(GTK_WINDOW(ScannerWindow),FALSE,FALSE,TRUE);
-    if (SCAN_WINDOW_ON_TOP)
+    if (SCANNER_WINDOW_ON_TOP)
         gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),GTK_WINDOW(MainWindow));
 
     /* The init position is define below, cause the scanner window must be showed before
@@ -947,40 +1017,41 @@ void Open_ScannerWindow (void)
     /* The menu */
     Menu = gtk_menu_new();
     /* Option for Tag */
-    MenuItem = gtk_menu_item_new_with_label(_(Scanner_Option_Menu_Items[FILL_TAG]));
+    MenuItem = gtk_menu_item_new_with_label(_(Scanner_Option_Menu_Items[SCANNER_FILL_TAG]));
     gtk_menu_append(GTK_MENU(Menu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Scanner_Option_Menu_Activate_Item,NULL);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Scanner_Option_Menu_Activate_Item,GINT_TO_POINTER(SCANNER_FILL_TAG));
     /* Option for FileName */
-    MenuItem = gtk_menu_item_new_with_label(_(Scanner_Option_Menu_Items[RENAME_FILE]));
+    MenuItem = gtk_menu_item_new_with_label(_(Scanner_Option_Menu_Items[SCANNER_RENAME_FILE]));
     gtk_menu_append(GTK_MENU(Menu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Scanner_Option_Menu_Activate_Item,NULL);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Scanner_Option_Menu_Activate_Item,GINT_TO_POINTER(SCANNER_RENAME_FILE));
     /* Option for ProcessFields */
-    MenuItem = gtk_menu_item_new_with_label(_(Scanner_Option_Menu_Items[PROCESS_FIELDS]));
+    MenuItem = gtk_menu_item_new_with_label(_(Scanner_Option_Menu_Items[SCANNER_PROCESS_FIELDS]));
     gtk_menu_append(GTK_MENU(Menu),MenuItem);
-    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Scanner_Option_Menu_Activate_Item,NULL);
+    gtk_signal_connect(GTK_OBJECT(MenuItem),"activate",(GtkSignalFunc)Scanner_Option_Menu_Activate_Item,GINT_TO_POINTER(SCANNER_PROCESS_FIELDS));
 
     gtk_option_menu_set_menu(GTK_OPTION_MENU(ScannerOptionMenu),Menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(ScannerOptionMenu),0);
+    // Selection of the item made at the end of the function
+    gtk_option_menu_set_history(GTK_OPTION_MENU(ScannerOptionMenu),scanner_type);
     gtk_tooltips_set_tip(Tips,ScannerOptionMenu,_("Select the type of scanner to use"),NULL);
 
 
     /* 'Scan file' button */
-    Button = gtk_button_new();
+    SWScanButton = gtk_button_new();
     Icon = Create_Pixmap_Icon(scan_xpm);
-    gtk_container_add(GTK_CONTAINER(Button),Icon);
-    gtk_box_pack_start(GTK_BOX(HBox1),Button,FALSE,FALSE,0);
-    gtk_button_set_relief(GTK_BUTTON(Button),GTK_RELIEF_NONE);
-    gtk_tooltips_set_tip(Tips,Button,_("Scan this file"),NULL);
-    gtk_signal_connect(GTK_OBJECT(Button),"clicked",(GtkSignalFunc)Action_Scan_File,NULL);
+    gtk_container_add(GTK_CONTAINER(SWScanButton),Icon);
+    gtk_box_pack_start(GTK_BOX(HBox1),SWScanButton,FALSE,FALSE,0);
+    gtk_button_set_relief(GTK_BUTTON(SWScanButton),GTK_RELIEF_NONE);
+    gtk_tooltips_set_tip(Tips,SWScanButton,_("Scan this file"),NULL);
+    gtk_signal_connect(GTK_OBJECT(SWScanButton),"clicked",(GtkSignalFunc)Action_Scan_File,NULL);
 
     /* 'Scan all files' button */
-    Button = gtk_button_new();
+    SWScanAllButton = gtk_button_new();
     Icon = Create_Pixmap_Icon(scan_all_xpm);
-    gtk_container_add(GTK_CONTAINER(Button),Icon);
-    gtk_box_pack_start(GTK_BOX(HBox1),Button,FALSE,FALSE,0);
-    gtk_button_set_relief(GTK_BUTTON(Button),GTK_RELIEF_NONE);
-    gtk_tooltips_set_tip(Tips,Button,_("Scan all files"),NULL);
-    gtk_signal_connect(GTK_OBJECT(Button),"clicked",(GtkSignalFunc)Action_Scan_All_Files,NULL);
+    gtk_container_add(GTK_CONTAINER(SWScanAllButton),Icon);
+    gtk_box_pack_start(GTK_BOX(HBox1),SWScanAllButton,FALSE,FALSE,0);
+    gtk_button_set_relief(GTK_BUTTON(SWScanAllButton),GTK_RELIEF_NONE);
+    gtk_tooltips_set_tip(Tips,SWScanAllButton,_("Scan all files"),NULL);
+    gtk_signal_connect(GTK_OBJECT(SWScanAllButton),"clicked",(GtkSignalFunc)Action_Scan_All_Files,NULL);
 
     /* Separator line */
     Separator = gtk_vseparator_new();
@@ -1023,7 +1094,6 @@ void Open_ScannerWindow (void)
     gtk_tooltips_set_tip(Tips,Button,_("Close this window"),NULL);
     gtk_signal_connect(GTK_OBJECT(Button),"clicked",(GtkSignalFunc)ScannerWindow_Quit,NULL);
 
-
     /*
      * Frame for Scan Tag
      */
@@ -1052,6 +1122,7 @@ void Open_ScannerWindow (void)
 
     /* Load masks into the combobox from a file */
     MasksList = Load_Scan_Tag_Masks_List();
+
     /* No mask read => load default masks */
     if (MasksList==NULL)
     {
@@ -1066,10 +1137,6 @@ void Open_ScannerWindow (void)
     gtk_box_pack_start(GTK_BOX(HBox2),MaskStatusIconBox,FALSE,FALSE,0);
     gtk_tooltips_set_tip(Tips,MaskStatusIconBox,_("Invalid Scanner Mask"),NULL);
 
-    /* Initial state of this frame */
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
-        gtk_widget_show(ScanTagFrame);
-
 
 
     /*
@@ -1124,10 +1191,6 @@ void Open_ScannerWindow (void)
     gtk_widget_show(RenameFilePreviewLabel);
     gtk_box_pack_start(GTK_BOX(vbox),RenameFilePreviewLabel,TRUE,TRUE,0);
 
-    /* Initial state of this frame */
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
-        gtk_widget_show(RenameFileFrame);
-
 
 
     /*
@@ -1220,18 +1283,18 @@ void Open_ScannerWindow (void)
 
     /* Group: capitalize, ... */
     ProcessFieldsAllUppercase = gtk_check_button_new_with_label         (_("All uppercase"));
-    ProcessFieldsAllDowncase = gtk_check_button_new_with_label          (_("All downcase"));
-    ProcessFieldsFirstLetterUppercase = gtk_check_button_new_with_label (_("First letter uppercase"));
+    ProcessFieldsAllDowncase  = gtk_check_button_new_with_label         (_("All downcase"));
+    ProcessFieldsFirstLetterUppercase  = gtk_check_button_new_with_label(_("First letter uppercase"));
     ProcessFieldsFirstLettersUppercase = gtk_check_button_new_with_label(_("First letter uppercase of each word"));
-    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsAllUppercase,        FALSE,FALSE,0);
-    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsAllDowncase,        FALSE,FALSE,0);
-    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsFirstLetterUppercase,    FALSE,FALSE,0);
-    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsFirstLettersUppercase,    FALSE,FALSE,0);
+    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsAllUppercase,         FALSE,FALSE,0);
+    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsAllDowncase,          FALSE,FALSE,0);
+    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsFirstLetterUppercase, FALSE,FALSE,0);
+    gtk_box_pack_start(GTK_BOX(VBox),ProcessFieldsFirstLettersUppercase,FALSE,FALSE,0);
     /* List creation for check buttons in group */
-    pf_cb_group2 = g_list_append (pf_cb_group2,ProcessFieldsAllUppercase);
-    pf_cb_group2 = g_list_append (pf_cb_group2,ProcessFieldsAllDowncase);
-    pf_cb_group2 = g_list_append (pf_cb_group2,ProcessFieldsFirstLetterUppercase);
-    pf_cb_group2 = g_list_append (pf_cb_group2,ProcessFieldsFirstLettersUppercase);
+    pf_cb_group2 = g_list_append(pf_cb_group2,ProcessFieldsAllUppercase);
+    pf_cb_group2 = g_list_append(pf_cb_group2,ProcessFieldsAllDowncase);
+    pf_cb_group2 = g_list_append(pf_cb_group2,ProcessFieldsFirstLetterUppercase);
+    pf_cb_group2 = g_list_append(pf_cb_group2,ProcessFieldsFirstLettersUppercase);
     /* Toggled signals */
     gtk_signal_connect(GTK_OBJECT(ProcessFieldsAllUppercase),"toggled",
         Process_Fields_Check_Button_Toggled,pf_cb_group2);
@@ -1244,10 +1307,8 @@ void Open_ScannerWindow (void)
     /* Set check buttons to init value */
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ProcessFieldsAllUppercase),PF_CONVERT_ALL_UPPERCASE);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ProcessFieldsAllDowncase),PF_CONVERT_ALL_DOWNCASE);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ProcessFieldsFirstLetterUppercase),
-        PF_CONVERT_FIRST_LETTER_UPPERCASE);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ProcessFieldsFirstLettersUppercase),
-        PF_CONVERT_FIRST_LETTERS_UPPERCASE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ProcessFieldsFirstLetterUppercase),PF_CONVERT_FIRST_LETTER_UPPERCASE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ProcessFieldsFirstLettersUppercase),PF_CONVERT_FIRST_LETTERS_UPPERCASE);
     /* Tooltips */
     gtk_tooltips_set_tip(Tips,ProcessFieldsAllUppercase,
         _("Convert all words in all fields to upper case. "
@@ -1299,10 +1360,6 @@ void Open_ScannerWindow (void)
         _("Duplicated spaces or underscores are removed. "
           "Example, before: 'Text__In__An   Entry', after: 'Text_In_An Entry'."),NULL);
 
-    /* Initial state of this frame */
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[PROCESS_FIELDS]))
-        gtk_widget_show(ProcessFieldsFrame);
-
 
 
     /*
@@ -1369,7 +1426,7 @@ void Open_ScannerWindow (void)
     gtk_clist_set_selection_mode(GTK_CLIST(MaskEditorClist),GTK_SELECTION_EXTENDED);
     gtk_signal_connect_after(GTK_OBJECT(MaskEditorClist),"select-row",
         (GtkSignalFunc)Mask_Editor_Clist_Select_Row,NULL);
-    gtk_signal_connect(GTK_OBJECT(MaskEditorClist), "key-press-event",
+    gtk_signal_connect(GTK_OBJECT(MaskEditorClist),"key-press-event",
         (GtkSignalFunc)Mask_Editor_Clist_Key_Press, NULL);
 
     /* The entry */
@@ -1478,6 +1535,9 @@ void Open_ScannerWindow (void)
     gtk_signal_emit_by_name(GTK_OBJECT(GTK_COMBO(ScannerRenameFileMaskCombo)->entry),"changed");
     gtk_signal_emit_by_name(GTK_OBJECT(LegendButton),"toggled");        /* To hide legend frame */
     gtk_signal_emit_by_name(GTK_OBJECT(MaskEditorButton),"toggled");    /* To hide mask editor frame */
+
+    // Activate the current menu in the option menu
+    Scanner_Option_Menu_Activate_Item(NULL,scanner_type);
 }
 void ScannerWindow_Key_Press (GtkWidget *window, GdkEvent *event)
 {
@@ -1504,21 +1564,21 @@ void Scan_Select_Mode_And_Run_Scanner (void)
 {
     if (!ScannerWindow && !ETFileList) return;
 
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
+    if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_FILL_TAG))
     {
         /* Set the default text to comment for tag scanner */
         if (SET_DEFAULT_COMMENT && (OVERWRITE_TAG_FIELD || strlen(gtk_entry_get_text_1(CommentEntry))==0 ) )
             gtk_entry_set_text(GTK_ENTRY(CommentEntry),DEFAULT_COMMENT);
 
-        /* Scanner Type: Scan Tag */
+        /* Run Scanner Type: Scan Tag */
         Scan_Tag_With_Mask((ET_File *)ETFileList->data);
-    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
+    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_RENAME_FILE))
     {
-        /* Scanner Type: Rename File */
+        /* Run Scanner Type: Rename File */
         Scan_Rename_File_With_Mask((ET_File *)ETFileList->data);
-    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[PROCESS_FIELDS]))
+    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_PROCESS_FIELDS))
     {
-        /* Scanner Type: Process Fields */
+        /* Run Scanner Type: Process Fields */
         Scan_Process_Fields((ET_File *)ETFileList->data);
     }
 }
@@ -1532,9 +1592,12 @@ void ScannerWindow_Quit (void)
         Scan_Apply_Changes();
 
         gtk_widget_destroy(ScannerWindow);
-        ScannerWindow = (GtkWidget *)NULL;
+        ScannerWindow     = (GtkWidget *)NULL;
+        ScannerOptionMenu = (GtkWidget *)NULL;
+        SWScanButton      = (GtkWidget *)NULL;
+        SWScanAllButton   = (GtkWidget *)NULL;
 
-        /* To avoid crashs after tests */
+        // To avoid crashs after tests
         ScannerScanTagMaskCombo    = (GtkWidget *)NULL;
         MaskStatusIconBox          = (GtkWidget *)NULL;
         ScannerRenameFileMaskCombo = (GtkWidget *)NULL;
@@ -1685,10 +1748,10 @@ void Scan_Check_Mask (GtkObject *wid_emit)
 
 
     /* Select and get result of check scanner */
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
+    if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_FILL_TAG))
     {
         hide_it = Scan_Check_Scan_Tag_Mask(mask);
-    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
+    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_RENAME_FILE))
     {
         hide_it = Scan_Check_Rename_File_Mask(mask);
     }else
@@ -1921,7 +1984,7 @@ void Mask_Editor_Clist_Add (void)
     gchar *text = NULL;
 
 
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
+    if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_FILL_TAG))
     {
         for (i=0;i<(sizeof(Scan_Masks)/sizeof(Scan_Masks[0]));i++)
         {
@@ -1931,7 +1994,7 @@ void Mask_Editor_Clist_Add (void)
         }
     }
 
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
+    if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_RENAME_FILE))
     {
         for (i=0;i<(sizeof(Rename_File_Masks)/sizeof(Rename_File_Masks[0]));i++)
         {
@@ -2016,7 +2079,7 @@ void Mask_Editor_Clist_Move_Down (void)
 
 void Mask_Editor_Clist_Save_Button (void)
 {
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
+    if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_FILL_TAG))
     {
         Mask_Editor_Create_Masks_List_From_Clist();
         Save_Scan_Tag_Masks_List(MasksList);
@@ -2024,7 +2087,7 @@ void Mask_Editor_Clist_Save_Button (void)
         if (!MasksList)
             MasksList = g_list_append(MasksList,"");
         gtk_combo_set_popdown_strings(GTK_COMBO(ScannerScanTagMaskCombo),MasksList);
-    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
+    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_RENAME_FILE))
     {
         Mask_Editor_Create_Masks_List_From_Clist();
         Save_Rename_File_Masks_List(RenameFileMasksList);
@@ -2078,7 +2141,7 @@ void Mask_Editor_Create_Masks_List_From_Clist (void)
 
 
     /* Recreate list */
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
+    if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_FILL_TAG))
     {
         MasksList = NULL;
         for (row=0;row<((GtkCList *)MaskEditorClist)->rows;row++)
@@ -2087,7 +2150,7 @@ void Mask_Editor_Create_Masks_List_From_Clist (void)
             if (text!=NULL)
                 MasksList = g_list_append(MasksList,g_strdup(text));
         }
-    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
+    }else if (Scanner_Option_Menu_Get_Active_Menu_Item(SCANNER_RENAME_FILE))
     {
         RenameFileMasksList = NULL;
         for (row=0;row<((GtkCList *)MaskEditorClist)->rows;row++)
@@ -2167,9 +2230,9 @@ void Scan_Apply_Changes (void)
         PF_CONVERT_FIRST_LETTER_UPPERCASE  = GTK_TOGGLE_BUTTON(ProcessFieldsFirstLetterUppercase)->active;
         PF_CONVERT_FIRST_LETTERS_UPPERCASE = GTK_TOGGLE_BUTTON(ProcessFieldsFirstLettersUppercase)->active;
         /* Group: remove/insert space */
-        PF_REMOVE_SPACE    = GTK_TOGGLE_BUTTON(ProcessFieldsConvertIntoSpace)->active;
-        PF_INSERT_SPACE    = GTK_TOGGLE_BUTTON(ProcessFieldsConvertSpace)->active;
-        PF_ONLY_ONE_SPACE  = GTK_TOGGLE_BUTTON(ProcessFieldsOnlyOneSpace)->active;
+        PF_REMOVE_SPACE   = GTK_TOGGLE_BUTTON(ProcessFieldsConvertIntoSpace)->active;
+        PF_INSERT_SPACE   = GTK_TOGGLE_BUTTON(ProcessFieldsConvertSpace)->active;
+        PF_ONLY_ONE_SPACE = GTK_TOGGLE_BUTTON(ProcessFieldsOnlyOneSpace)->active;
     }
 }
 
@@ -2187,68 +2250,72 @@ void Scan_Display_Info (void)
 
 
 /*
- * Returns 1 if item_text match with the current active menu item, else 0
+ * Returns 1 if item match with the current active option menu item, else returns 0.
  */
-gboolean Scanner_Option_Menu_Get_Active_Menu_Item (gchar *item_text)
+gboolean Scanner_Option_Menu_Get_Active_Menu_Item (Scanner_Option_Menu_Items_Value item)
 {
     GtkWidget *child;
     gchar *text;
 
-    if (GTK_BIN(ScannerOptionMenu)->child)
+    if (ScannerOptionMenu && GTK_BIN(ScannerOptionMenu)->child)
     {
-        child = GTK_BIN (ScannerOptionMenu)->child;
-        if (GTK_IS_LABEL (child))
-            gtk_label_get (GTK_LABEL (child), &text);
+        child = GTK_BIN(ScannerOptionMenu)->child;
+        if (GTK_IS_LABEL(child))
+            gtk_label_get(GTK_LABEL(child), &text);
+        else
+            return 0;
+    }else
+    {
+        return 0;
+    }
+
+    if ( item >= 0 && item < (sizeof(Scanner_Option_Menu_Items)/sizeof(Scanner_Option_Menu_Items[0])) )
+    {
+        /* Do not forget gettext codes to enable the comparison */
+        if (strcmp(text,_(Scanner_Option_Menu_Items[item]))==0)
+            return 1;
         else
             return 0;
     }else
         return 0;
-
-    /* Do not forget gettext codes to enable the comparison */
-    if (strcmp(text,_(item_text))==0) return 1;
-    else                              return 0;
 }
 
 
 /*
  * Function when you select an item of the option menu
  */
-void Scanner_Option_Menu_Activate_Item (void)
+void Scanner_Option_Menu_Activate_Item (GtkWidget *widget, Scanner_Option_Menu_Items_Value item)
 {
-    //gtk_widget_set_sensitive(MaskEditorButton,TRUE);
-    //gtk_widget_set_sensitive(LegendButton,TRUE);
-    gtk_widget_show(MaskEditorButton);
-    gtk_widget_show(LegendButton);
-
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[FILL_TAG]))
+    switch (item)
     {
-        gtk_widget_show(ScanTagFrame);
-        gtk_widget_hide(RenameFileFrame);
-        gtk_widget_hide(ProcessFieldsFrame);
-        Mask_Editor_Clist_Load_Mask_List(MasksList);
-    }
-
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[RENAME_FILE]))
-    {
-        gtk_widget_show(RenameFileFrame);
-        gtk_widget_hide(ScanTagFrame);
-        gtk_widget_hide(ProcessFieldsFrame);
-        Mask_Editor_Clist_Load_Mask_List(RenameFileMasksList);
-        Scan_Rename_File_Generate_Preview();
-    }
-
-    if (Scanner_Option_Menu_Get_Active_Menu_Item(Scanner_Option_Menu_Items[PROCESS_FIELDS]))
-    {
-        gtk_widget_hide(ScanTagFrame);
-        gtk_widget_hide(RenameFileFrame);
-        gtk_widget_show_all(ProcessFieldsFrame);
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(MaskEditorButton),FALSE);
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(LegendButton),FALSE);
-        Mask_Editor_Clist_Load_Mask_List(NULL);
-        //gtk_widget_set_sensitive(MaskEditorButton,FALSE);
-        //gtk_widget_set_sensitive(LegendButton,FALSE);
-        gtk_widget_hide(MaskEditorButton);
-        gtk_widget_hide(LegendButton);
+        case SCANNER_FILL_TAG:
+            gtk_widget_show(MaskEditorButton);
+            gtk_widget_show(LegendButton);
+            gtk_widget_show(ScanTagFrame);
+            gtk_widget_hide(RenameFileFrame);
+            gtk_widget_hide(ProcessFieldsFrame);
+            Mask_Editor_Clist_Load_Mask_List(MasksList);
+            break;
+        case SCANNER_RENAME_FILE:
+            gtk_widget_show(MaskEditorButton);
+            gtk_widget_show(LegendButton);
+            gtk_widget_hide(ScanTagFrame);
+            gtk_widget_show(RenameFileFrame);
+            gtk_widget_hide(ProcessFieldsFrame);
+            Mask_Editor_Clist_Load_Mask_List(RenameFileMasksList);
+            Scan_Rename_File_Generate_Preview();
+            break;
+        case SCANNER_PROCESS_FIELDS:
+            gtk_widget_hide(MaskEditorButton);
+            gtk_widget_hide(LegendButton);
+            gtk_widget_hide(ScanTagFrame);
+            gtk_widget_hide(RenameFileFrame);
+            gtk_widget_show_all(ProcessFieldsFrame);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(MaskEditorButton),FALSE);
+            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(LegendButton),FALSE);
+            Mask_Editor_Clist_Load_Mask_List(NULL);
+            break;
+        
     }
 }
 
