@@ -1,7 +1,7 @@
 /* config.c - 2000/06/21 */
 /*
  *  EasyTAG - Tag editor for MP3 and OGG files
- *  Copyright (C) 2000  Jerome Couderc <j.couderc@ifrance.com>
+ *  Copyright (C) 2000-2002  Jerome Couderc <j.couderc@ifrance.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -44,26 +44,31 @@
  * Declaration *
  ***************/
 
-/* Base directory created into home dir */
+// Base directory created into home dir 
 gchar *EASYTAG_DIR                          = ".easytag/";
-/* File for configuration */
+// File for configuration
 gchar *CONFIG_FILE                          = ".easytag/easytagrc";
-/* File of masks for tag scanner */
+// File of masks for tag scanner
 gchar *SCAN_TAG_MASKS_FILE                  = ".easytag/scan_tag.mask";
-/* File of masks for rename file scanner */
+// File of masks for rename file scanner
 gchar *RENAME_FILE_MASKS_FILE                = ".easytag/rename_file.mask";
-/* File for history of DefaultPathToMp3 combobox */
+// File for history of DefaultPathToMp3 combobox
 gchar *DEFAULT_PATH_TO_MP3_HISTORY_FILE      = ".easytag/default_path_to_mp3.history";
-/* File for history of DefaultComment combobox */
+// File for history of DefaultComment combobox
 gchar *DEFAULT_TAG_COMMENT_HISTORY_FILE      = ".easytag/default_tag_comment.history";
-/* File for history of TranslationTablePath combobox */
-gchar *TRANSLATION_TABLE_PATH_HISTORY_FILE   = ".easytag/translation_table_path.history";
-/* File for history of BrowserEntry combobox */
+// File for history of BrowserEntry combobox
 gchar *PATH_ENTRY_HISTORY_FILE               = ".easytag/browser_path.history";
-/* File for history of PlayListNameEntry combobox */
+// File for history of PlayListNameEntry combobox
 gchar *PLAY_LIST_NAME_HISTORY_FILE           = ".easytag/play_list_name.history";
-/* File for history of run program combobox */
+// File for history of run program combobox
 gchar *RUN_PROGRAM_HISTORY_FILE              = ".easytag/run_program.history";
+// File for history of run player combobox
+gchar *AUDIO_FILE_PLAYER_HISTORY_FILE        = ".easytag/audio_file_player.history";
+// File for history of search string combobox
+gchar *SEARCH_FILE_HISTORY_FILE              = ".easytag/search_file.history";
+// File for history of FileToLoad combobox
+gchar *FILE_TO_LOAD_HISTORY_FILE             = ".easytag/file_to_load.history";
+
 
 
 /**************
@@ -79,7 +84,8 @@ tConfigVariable Config_Variables[] =
 {
     {"load_on_startup",                CV_TYPE_BOOL,    &LOAD_ON_STARTUP            },
     {"default_path_to_mp3",            CV_TYPE_STRING,  &DEFAULT_PATH_TO_MP3        },
-    {"sorting_file_mode",              CV_TYPE_BOOL,    &SORTING_FILE_MODE          },
+    {"browser_line_style",             CV_TYPE_BOOL,    &BROWSER_LINE_STYLE         },
+    {"browser_expander_style",         CV_TYPE_BOOL,    &BROWSER_EXPANDER_STYLE     },
     {"write_id3v1_tag",                CV_TYPE_BOOL,    &WRITE_ID3V1_TAG            },
     {"write_id3v2_tag",                CV_TYPE_BOOL,    &WRITE_ID3V2_TAG            },
     {"strip_tag_when_empty_fields",    CV_TYPE_BOOL,    &STRIP_TAG_WHEN_EMPTY_FIELDS},
@@ -96,31 +102,14 @@ tConfigVariable Config_Variables[] =
     {"set_pane_handle_position2",      CV_TYPE_INT,     &SET_PANE_HANDLE_POSITION2  },
     {"pane_handle_position2",          CV_TYPE_INT,     &PANE_HANDLE_POSITION2      },
     {"show_header_infos",              CV_TYPE_BOOL,    &SHOW_HEADER_INFO           },
-    {"show_command_buttons",           CV_TYPE_BOOL,    &SHOW_COMMAND_BUTTONS       },
+    {"use_colored_tooltips",           CV_TYPE_BOOL,    &USE_COLORED_TOOLTIPS       },
 
-    {"show_spin_button",               CV_TYPE_BOOL,    &SHOW_SPIN_BUTTON        },
-    {"show_go_button",                 CV_TYPE_BOOL,    &SHOW_SPIN_BUTTON        },
-    {"show_first_button",              CV_TYPE_BOOL,    &SHOW_FIRST_BUTTON       },
-    {"show_prev_button",               CV_TYPE_BOOL,    &SHOW_PREV_BUTTON        },
-    {"show_next_button",               CV_TYPE_BOOL,    &SHOW_NEXT_BUTTON        },
-    {"show_last_button",               CV_TYPE_BOOL,    &SHOW_LAST_BUTTON        },
-    {"show_scan_button",               CV_TYPE_BOOL,    &SHOW_SCAN_BUTTON        },
-    {"show_scan_all_button",           CV_TYPE_BOOL,    &SHOW_SCAN_ALL_BUTTON    },
-    {"show_remove_button",             CV_TYPE_BOOL,    &SHOW_REMOVE_BUTTON      },
-    {"show_remove_all_button",         CV_TYPE_BOOL,    &SHOW_REMOVE_ALL_BUTTON  },
-    {"show_undo_button",               CV_TYPE_BOOL,    &SHOW_UNDO_BUTTON        },
-    {"show_undo_all_button",           CV_TYPE_BOOL,    &SHOW_UNDO_ALL_BUTTON    },
-    {"show_redo_button",               CV_TYPE_BOOL,    &SHOW_REDO_BUTTON        },
-    {"show_redo_all_button",           CV_TYPE_BOOL,    &SHOW_REDO_ALL_BUTTON    },
-    {"show_save_button",               CV_TYPE_BOOL,    &SHOW_SAVE_BUTTON        },
-    {"show_save_all_button",           CV_TYPE_BOOL,    &SHOW_SAVE_ALL_BUTTON    },
-    {"show_quit_button",               CV_TYPE_BOOL,    &SHOW_QUIT_BUTTON        },
-
-    {"use_colored_tooltips",           CV_TYPE_BOOL,    &USE_COLORED_TOOLTIPS    },
-
-    {"date_auto_completion",                 CV_TYPE_BOOL,    &DATE_AUTO_COMPLETION           },
-    {"number_track_formated",                     CV_TYPE_BOOL,    &NUMBER_TRACK_FORMATED     },
     {"replace_illegal_character_in_filename",CV_TYPE_BOOL,    &REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME},
+    {"date_auto_completion",                 CV_TYPE_BOOL,    &DATE_AUTO_COMPLETION           },
+    {"number_track_formated",                CV_TYPE_BOOL,    &NUMBER_TRACK_FORMATED          },
+    {"ogg_tag_write_xmms_comment",           CV_TYPE_BOOL,    &OGG_TAG_WRITE_XMMS_COMMENT     },
+    {"sorting_file_mode",                    CV_TYPE_BOOL,    &SORTING_FILE_MODE              },
+    {"sorting_file_case_sensitive",          CV_TYPE_BOOL,    &SORTING_FILE_CASE_SENSITIVE    },
 
     {"use_character_set_translation",        CV_TYPE_BOOL,    &USE_CHARACTER_SET_TRANSLATION  },
     {"file_character_set",                   CV_TYPE_STRING,  &FILE_CHARACTER_SET             },
@@ -130,12 +119,15 @@ tConfigVariable Config_Variables[] =
     {"message_box_position_center",          CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_CENTER    },
     {"message_box_position_mouse",           CV_TYPE_BOOL,    &MESSAGE_BOX_POSITION_MOUSE     },
 
+    {"audio_file_player",                    CV_TYPE_STRING,  &AUDIO_FILE_PLAYER              },
+
     {"convert_underscore_and_p20_into_space",CV_TYPE_BOOL,    &CONVERT_UNDERSCORE_AND_P20_INTO_SPACE},
     {"convert_space_into_underscore",        CV_TYPE_BOOL,    &CONVERT_SPACE_INTO_UNDERSCORE        },
     {"overwrite_tag_field",                  CV_TYPE_BOOL,    &OVERWRITE_TAG_FIELD                  },
     {"set_default_comment",                  CV_TYPE_BOOL,    &SET_DEFAULT_COMMENT                  },
     {"default_comment",                      CV_TYPE_STRING,  &DEFAULT_COMMENT                      },
-    {"scan_window_on_top",                   CV_TYPE_BOOL,    &SCAN_WINDOW_ON_TOP                   },
+    {"open_scanner_window_on_startup",       CV_TYPE_BOOL,    &OPEN_SCANNER_WINDOW_ON_STARTUP       },
+    {"scanner_window_on_top",                CV_TYPE_BOOL,    &SCANNER_WINDOW_ON_TOP                },
     {"set_scanner_window_position",          CV_TYPE_BOOL,    &SET_SCANNER_WINDOW_POSITION          },
     {"scanner_window_x",                     CV_TYPE_INT,     &SCANNER_WINDOW_X                     },
     {"scanner_window_y",                     CV_TYPE_INT,     &SCANNER_WINDOW_Y                     },
@@ -184,7 +176,8 @@ void Init_Config_Variables (void)
      */
     LOAD_ON_STARTUP              = 1;
     DEFAULT_PATH_TO_MP3          = g_strdup(HOME_VARIABLE);
-    SORTING_FILE_MODE            = SORTING_BY_ASCENDING_FILENAME;
+    BROWSER_LINE_STYLE           = GTK_CTREE_LINES_DOTTED;
+    BROWSER_EXPANDER_STYLE       = GTK_CTREE_EXPANDER_SQUARE;
     WRITE_ID3V1_TAG              = 1;
     WRITE_ID3V2_TAG              = 1;
     STRIP_TAG_WHEN_EMPTY_FIELDS  = 1;
@@ -204,27 +197,7 @@ void Init_Config_Variables (void)
     SET_PANE_HANDLE_POSITION2 = 1;
     PANE_HANDLE_POSITION2     = 230;
     SHOW_HEADER_INFO          = 1;
-    SHOW_COMMAND_BUTTONS      = 0;
-
-    SHOW_SPIN_BUTTON       = 1;
-    SHOW_GO_BUTTON         = 1;
-    SHOW_FIRST_BUTTON      = 1;
-    SHOW_PREV_BUTTON       = 1;
-    SHOW_NEXT_BUTTON       = 1;
-    SHOW_LAST_BUTTON       = 1;
-    SHOW_SCAN_BUTTON       = 1;
-    SHOW_SCAN_ALL_BUTTON   = 1;
-    SHOW_REMOVE_BUTTON     = 1;
-    SHOW_REMOVE_ALL_BUTTON = 1;
-    SHOW_UNDO_BUTTON       = 1;
-    SHOW_UNDO_ALL_BUTTON   = 1;
-    SHOW_REDO_BUTTON       = 1;
-    SHOW_REDO_ALL_BUTTON   = 1;
-    SHOW_SAVE_BUTTON       = 1;
-    SHOW_SAVE_ALL_BUTTON   = 1;
-    SHOW_QUIT_BUTTON       = 1;
-    
-    USE_COLORED_TOOLTIPS   = 0;
+    USE_COLORED_TOOLTIPS      = 0;
 
     /*
      * Scanner
@@ -234,7 +207,8 @@ void Init_Config_Variables (void)
     OVERWRITE_TAG_FIELD                   = 0;
     SET_DEFAULT_COMMENT                   = 0;
     DEFAULT_COMMENT                       = g_strdup("Tagged with EasyTAG");
-    SCAN_WINDOW_ON_TOP                    = 1;
+    OPEN_SCANNER_WINDOW_ON_STARTUP        = 1;
+    SCANNER_WINDOW_ON_TOP                 = 1;
     SET_SCANNER_WINDOW_POSITION           = 0;
     SCANNER_WINDOW_X                      = 11;
     SCANNER_WINDOW_Y                      = 155;
@@ -245,8 +219,10 @@ void Init_Config_Variables (void)
     REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME = 1;
     DATE_AUTO_COMPLETION          = 1;
     NUMBER_TRACK_FORMATED         = 1;
+    OGG_TAG_WRITE_XMMS_COMMENT    = 1;
+    SORTING_FILE_MODE             = SORTING_BY_ASCENDING_FILENAME;
+    SORTING_FILE_CASE_SENSITIVE   = 0;
     USE_CHARACTER_SET_TRANSLATION = 0;
-    //TRANSLATION_TABLE_PATH      = NULL;
     FILE_CHARACTER_SET            = g_strdup("UTF-8");
     USER_CHARACTER_SET            = g_strdup("ISO-8859-15");
 
@@ -254,6 +230,8 @@ void Init_Config_Variables (void)
     MESSAGE_BOX_POSITION_CENTER   = 1;
     MESSAGE_BOX_POSITION_MOUSE    = 0;
 
+    AUDIO_FILE_PLAYER             = g_strdup("xmms -p");
+
     /*
      * Confirmation
      */
@@ -326,32 +304,14 @@ void Apply_Changes (void)
         SET_PANE_HANDLE_POSITION2 = GTK_TOGGLE_BUTTON(SetPaneHandlePosition2)->active;
         PANE_HANDLE_POSITION2     = atoi(gtk_entry_get_text_1(PaneHandlePosition2));
         SHOW_HEADER_INFO          = GTK_TOGGLE_BUTTON(ShowHeaderInfos)->active;
-        SHOW_COMMAND_BUTTONS      = GTK_TOGGLE_BUTTON(ShowCommandButtons)->active;
-
-        SHOW_SPIN_BUTTON        = GTK_TOGGLE_BUTTON(ShowSpinButton)->active;
-        SHOW_GO_BUTTON          = GTK_TOGGLE_BUTTON(ShowGoButton)->active;
-        SHOW_FIRST_BUTTON       = GTK_TOGGLE_BUTTON(ShowFirstButton)->active;
-        SHOW_PREV_BUTTON        = GTK_TOGGLE_BUTTON(ShowPrevButton)->active;
-        SHOW_NEXT_BUTTON        = GTK_TOGGLE_BUTTON(ShowNextButton)->active;
-        SHOW_LAST_BUTTON        = GTK_TOGGLE_BUTTON(ShowLastButton)->active;
-        SHOW_SCAN_BUTTON        = GTK_TOGGLE_BUTTON(ShowScanButton)->active;
-        SHOW_SCAN_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowScanAllButton)->active;
-        SHOW_REMOVE_BUTTON      = GTK_TOGGLE_BUTTON(ShowRemoveButton)->active;
-        SHOW_REMOVE_ALL_BUTTON  = GTK_TOGGLE_BUTTON(ShowRemoveAllButton)->active;
-        SHOW_UNDO_BUTTON        = GTK_TOGGLE_BUTTON(ShowUndoButton)->active;
-        SHOW_UNDO_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowUndoAllButton)->active;
-        SHOW_REDO_BUTTON        = GTK_TOGGLE_BUTTON(ShowRedoButton)->active;
-        SHOW_REDO_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowRedoAllButton)->active;
-        SHOW_SAVE_BUTTON        = GTK_TOGGLE_BUTTON(ShowSaveButton)->active;
-        SHOW_SAVE_ALL_BUTTON    = GTK_TOGGLE_BUTTON(ShowSaveAllButton)->active;
-        SHOW_QUIT_BUTTON        = GTK_TOGGLE_BUTTON(ShowQuitButton)->active;
-        
-        USE_COLORED_TOOLTIPS    = GTK_TOGGLE_BUTTON(UseColoredTooltips)->active;
+        USE_COLORED_TOOLTIPS      = GTK_TOGGLE_BUTTON(UseColoredTooltips)->active;
 
         /* Misc */
         REPLACE_ILLEGAL_CHARACTERS_IN_FILENAME = GTK_TOGGLE_BUTTON(ReplaceIllegalCharactersInFilename)->active;
         DATE_AUTO_COMPLETION          = GTK_TOGGLE_BUTTON(DateAutoCompletion)->active;
         NUMBER_TRACK_FORMATED         = GTK_TOGGLE_BUTTON(NumberTrackFormated)->active;
+        OGG_TAG_WRITE_XMMS_COMMENT    = GTK_TOGGLE_BUTTON(OggTagWriteXmmsComment)->active;
+        SORTING_FILE_CASE_SENSITIVE   = GTK_TOGGLE_BUTTON(SortingFileCaseSensitive)->active;
         USE_CHARACTER_SET_TRANSLATION = GTK_TOGGLE_BUTTON(UseCharacterSetTranslation)->active;
         FILE_CHARACTER_SET     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(FileCharacterSetEntry));
         USER_CHARACTER_SET     = Charset_Get_Name_From_Title(gtk_entry_get_text_1(UserCharacterSetEntry));
@@ -360,6 +320,8 @@ void Apply_Changes (void)
         MESSAGE_BOX_POSITION_CENTER = GTK_TOGGLE_BUTTON(MessageBoxPositionCenter)->active;
         MESSAGE_BOX_POSITION_MOUSE  = GTK_TOGGLE_BUTTON(MessageBoxPositionMouse)->active;
 
+        AUDIO_FILE_PLAYER           = g_strdup(gtk_entry_get_text_1(FilePlayerEntry));
+
         /* Scanner */
         CONVERT_UNDERSCORE_AND_P20_INTO_SPACE = GTK_TOGGLE_BUTTON(ConvertUnderscoreAndP20IntoSpace)->active;
         CONVERT_SPACE_INTO_UNDERSCORE         = GTK_TOGGLE_BUTTON(ConvertSpaceIntoUnderscore)->active;
@@ -368,10 +330,11 @@ void Apply_Changes (void)
         SET_DEFAULT_COMMENT = GTK_TOGGLE_BUTTON(SetDefaultComment)->active;
         DEFAULT_COMMENT     = g_strdup(gtk_entry_get_text_1(DefaultComment));
 
-        SCAN_WINDOW_ON_TOP          = GTK_TOGGLE_BUTTON(ScanWindowOnTop)->active;
-        SET_SCANNER_WINDOW_POSITION = GTK_TOGGLE_BUTTON(SetScannerWindowPosition)->active;
-        SCANNER_WINDOW_X            = atoi(gtk_entry_get_text_1(ScannerWindowX));
-        SCANNER_WINDOW_Y            = atoi(gtk_entry_get_text_1(ScannerWindowY));
+        OPEN_SCANNER_WINDOW_ON_STARTUP = GTK_TOGGLE_BUTTON(OpenScannerWindowOnStartup)->active;
+        SCANNER_WINDOW_ON_TOP          = GTK_TOGGLE_BUTTON(ScannerWindowOnTop)->active;
+        SET_SCANNER_WINDOW_POSITION    = GTK_TOGGLE_BUTTON(SetScannerWindowPosition)->active;
+        SCANNER_WINDOW_X               = atoi(gtk_entry_get_text_1(ScannerWindowX));
+        SCANNER_WINDOW_Y               = atoi(gtk_entry_get_text_1(ScannerWindowY));
 
         /* Confirmation */
         CONFIRM_BEFORE_EXIT    = GTK_TOGGLE_BUTTON(ConfirmBeforeExit)->active;
@@ -390,18 +353,13 @@ void Apply_Changes (void)
         if (SHOW_HEADER_INFO) gtk_widget_show_all(HeaderInfosTable);
         else                  gtk_widget_hide_all(HeaderInfosTable);
 
-        if (SHOW_COMMAND_BUTTONS) gtk_widget_show_all(CommandArea);
-        else                      gtk_widget_hide_all(CommandArea);
-        /* Must be placed after "SHOW_COMMAND_BUTTONS" cause it can show ALL */
-        Update_Button_Visibility_Into_Command_Bar();
-
         /* Update state of check-menu-item into main/popup menu to browse subdirs */
         Check_Menu_Item_Update_Browse_Subdir();
 
         if (ScannerWindow)
         {
             gtk_widget_hide(ScannerWindow);
-            if (SCAN_WINDOW_ON_TOP)
+            if (SCANNER_WINDOW_ON_TOP)
                 gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),GTK_WINDOW(MainWindow));
             else
                 gtk_window_set_transient_for(GTK_WINDOW(ScannerWindow),NULL);
@@ -574,7 +532,7 @@ void Read_Config (void)
     FILE *file;
     gchar buffer[MAX_STRING_LEN];
 
-    /* The file to write */
+    /* The file to read */
     if (!HOME_VARIABLE) return;
     file_path = g_strconcat(HOME_VARIABLE,
                             HOME_VARIABLE[strlen(HOME_VARIABLE)-1]!='/'?"/":"",
@@ -785,18 +743,6 @@ void Save_Default_Tag_Comment_Text_List (GList *list)
     Save_Glist_To_File(list,DEFAULT_TAG_COMMENT_HISTORY_FILE);
 }
 
-/*
- * Functions for writing and reading list of 'TranslationTablePath' combobox
- */
-GList *Load_Translation_Table_Path_List (void)
-{
-    return Create_Glist_From_File(TRANSLATION_TABLE_PATH_HISTORY_FILE);
-}
-void Save_Translation_Table_Path_List (GList *list)
-{
-    Save_Glist_To_File(list,TRANSLATION_TABLE_PATH_HISTORY_FILE);
-}
-
 /*
  * Functions for writing and reading list of 'BrowserEntry' combobox
  */
@@ -833,6 +779,44 @@ void Save_Run_Program_List (GList *list)
     Save_Glist_To_File(list,RUN_PROGRAM_HISTORY_FILE);
 }
 
+/*
+ * Functions for writing and reading list of combobox to run file audio player
+ */
+GList *Load_Audio_File_Player_List (void)
+{
+    return Create_Glist_From_File(AUDIO_FILE_PLAYER_HISTORY_FILE);
+}
+void Save_Audio_File_Player_List (GList *list)
+{
+    Save_Glist_To_File(list,AUDIO_FILE_PLAYER_HISTORY_FILE);
+}
+
+/*
+ * Functions for writing and reading list of combobox to search a string into file (tag or filename)
+ */
+GList *Load_Search_File_List (void)
+{
+    return Create_Glist_From_File(SEARCH_FILE_HISTORY_FILE);
+}
+void Save_Search_File_List (GList *list)
+{
+    Save_Glist_To_File(list,SEARCH_FILE_HISTORY_FILE);
+}
+
+/*
+ * Functions for writing and reading list of combobox of path of file to load to rename files
+ */
+GList *Load_File_To_Load_List (void)
+{
+    return Create_Glist_From_File(FILE_TO_LOAD_HISTORY_FILE);
+}
+void Save_File_To_Load_List (GList *list)
+{
+    Save_Glist_To_File(list,FILE_TO_LOAD_HISTORY_FILE);
+}
+
+
+
 
 
 /*
